/* ********************************************************
    dfmax.cpp

    Author = Hideyuki Harada
    Nihon Univ. Tani Research Group


    Basic Algorithm for resolving MaxClique problem
    designed for "g++ 3.3.3" with "LEDA 4.5"
   ******************************************************** */


#include<LEDA/graph.h>
#include<LEDA/list.h>
#include<time.h>
#include<iostream.h>

using namespace leda;

graph G;           // undirected graph

list<node> Qmax;   // node set for MaxClique

int Branch_count;  // number of counted SST branch


void read_graph(int argc, char *argv[]);   // read filename from command line

void initialize(list<node> *R, list<node> *Q);  // clear Q and put V to R

void show_result(float T, char *argv[]);  // write result

void dfmax(list<node> R, list<node> Q);  // basic algorithm


//*************************************************


int main(int argc, char *argv[]){

  float T = used_time();
  list<node> R, Q;

  if(argc != 2){
    cout << "usage: dfmax [file]" << endl;
    exit(1);
  }

  read_graph(argc, argv);

  initialize(&R, &Q);


  // start procedure
  dfmax(R, Q);


  show_result(T, argv);

  return 0;

}


void dfmax(list<node> R, list<node> Q){

  node p, v;
  list<node> Rp;

  while(!R.empty()){

    Branch_count++;

    p = R.head();

    Q.push(p);

    Rp.clear();
    // Rp = intersect(R, G.adj_nodes(p));
    forall_adj_nodes(v, p){
      if(R.search(v) != NULL){
	Rp.push(v);
      }
    }

    if(!Rp.empty()){
      dfmax(Rp, Q);
    }
    else{
      if(Q.size() > Qmax.size()){  // apply new MaxClique
	Qmax.clear();
	forall(v, Q){
	  Qmax.append(v);
	}
      }
    }
    
    Q.remove(p);
    R.remove(p);
  }

  return;

}


void read_graph(int argc, char *argv[]){

  G.read(argv[1]);
  G.make_undirected();

  return;
}

void initialize(list<node> *R, list<node> *Q){

  (*Q).clear();
  (*R) = G.all_nodes();
  Qmax.clear();

  return;
}

void show_result(float T, char *argv[]){

  node v;
  FILE *fp;

  if(fp = fopen((argv[1]+".result"), "w") == NULL){
    cout << "cant write into result file!";
    exit();
  }

  cout << "Filename = " << argv[1] << endl;
  cout << " Number of vertexes = " << G.number_of_nodes() << endl;
  cout << " Number of edges = " << G.number_of_edges() << endl;
  cout << "Used time = " << used_time(T) << endl;
  cout << "SST Branch count = " << Branch_count << endl;
  cout << "MAX CLIQUE SIZE = " << Qmax.size() << endl;
  cout << "CLIQUE ELEMENT(S) :" << endl;

  forall(v, Qmax){
    cout << " " << index(v);
  }

  cout << endl;

  fclose(fp);

  return;
}
